--- my first lua and love2d script
--- bouncing balls simulated by physics module

function love.load()
	width,height=love.graphics.getDimensions()
	math.randomseed(os.time())

	setupBall()
	setupEnemy()
	setupNature()	
	buildWorld()

	
	-- collision setting
	collision={}
	collision.timer=0
	collision.color={0,255,0}

	--spawnBall()
	
	
	
	--- boy:setGravityScale(1)
	--- boy:setActive(true)
	msg_move_down="moving down"
	msg_move_right="moving right"
	msg_greeting="hello,this is my first LOVE script"
	
end

function buildWorld()
	wall={}
	wall.width=width*0.95
	wall.height=height*0.02
	wall.top={}
	wall.bottom={}
	wall.left={}
	wall.right={}
	love.physics.setMeter(10)
	world=love.physics.newWorld(0,9.81*10,false)
	wall.top.body=love.physics.newBody(world,(width-wall.width)/2 +wall.width/2,wall.height/2,"static")
	wall.top.shape=love.physics.newRectangleShape(wall.width,wall.height)
	wall.top.fixture=love.physics.newFixture(wall.top.body,wall.top.shape,1)
	wall.bottom.body=love.physics.newBody(world,(width-wall.width)/2 +wall.width/2,height-wall.height/2,"static")
	wall.bottom.shape=love.physics.newRectangleShape(wall.width,wall.height)
	wall.bottom.fixture=love.physics.newFixture(wall.bottom.body,wall.bottom.shape,1)
	wall.left.body=love.physics.newBody(world,wall.height/2,(height-wall.width)/2+wall.width/2,"static")
	wall.left.shape=love.physics.newRectangleShape(wall.height,wall.width)
	wall.left.fixture=love.physics.newFixture(wall.left.body,wall.left.shape,1)
	wall.right.body=love.physics.newBody(world,width-wall.height/2,(height-wall.width)/2+wall.width/2,"static")
	wall.right.shape=love.physics.newRectangleShape(wall.height,wall.width)
	wall.right.fixture=love.physics.newFixture(wall.right.body,wall.right.shape,1)
	
	world:setCallbacks(beginContact, endContact, preSolve, postSolve)
end

function setupBall()
	ball={}
	ball.color={{64,64,64},{128,128,128},{192,192,192},{255,255,255}}
	ball.xvel=0
	ball.yvel=0
	ball.timer=0
	ball.maxnum=3
end

function setupEnemy()
	enemy={}
	enemy.color={{200,0,200},{0,200,200},{0,200,200},{0,0,255}}
	enemy.xvel=0
	enemy.yvel=0
	enemy.timer=0
	enemy.maxnum=5
end


function setupNature()
	wind={}
	wind.speed=50
	wind.dir=0
	wind.timer=0
end

function spawnEnemy()
	local pos={{width/5,height/3},{width*0.85,height/3}}
	---a={{1,4},{5,10}}
	local r=math.random(1,2)
	
	local x,y=unpack(pos[r])

	local body=love.physics.newBody(world,x,y,"dynamic")
	local shape=love.physics.newChainShape(true,0,10,5,0,10,10)
	local fixture=love.physics.newFixture(body,shape,1)
	fixture:setUserData({['id']='enemy',['health']=4})
	fixture:setRestitution(1.2)
	table.insert(enemy,{body=body,shape=shape,fixture=fixture})
	
end	

function spawnBall()
	local pos={{width*0.2,height/3},{width*0.5,height/3},{width*0.7,height/3},{width*0.9,height/3}}
	---a={{1,4},{5,10}}
	local r=math.random(1,4)
	
	local x,y=unpack(pos[r])

	local body=love.physics.newBody(world,x,y,"dynamic")
	local shape=love.physics.newCircleShape(20)
	local fixture=love.physics.newFixture(body,shape,1)
	fixture:setUserData({['id']='ball',['health']=4})
	fixture:setRestitution(0.98)
	table.insert(ball,{body=body,shape=shape,fixture=fixture})
	
	
end	

function NaturePhysics(dt)
	wind.timer=wind.timer+dt
	
	-- change wind direction
	if wind.timer>10 then
		wind.dir=math.random(-1,1)	
		wind.timer=0
	end
end

function EnemyPhysics(dt)
	enemy.timer=enemy.timer+dt
	
	-- generate a new enemy
	if enemy.timer>5 and #enemy < enemy.maxnum then
		spawnEnemy()
		enemy.timer=0
	end

	for i,v in ipairs(enemy) do
		v.xvel,v.yvel=v.body:getLinearVelocity()
		v.body:setLinearVelocity(v.xvel+wind.dir*wind.speed*dt,v.yvel)
		v.body:setAngularVelocity(wind.dir*1)
	end


end

function BallPhysics(dt)

	ball.timer=ball.timer+dt
	
	-- generate a new ball 
	if ball.timer>5 and #ball < ball.maxnum then
		spawnBall()
		ball.timer=0
	end

	for i,v in ipairs(ball) do
		v.xvel,v.yvel=v.body:getLinearVelocity()
		v.body:setLinearVelocity(v.xvel+wind.dir*wind.speed*dt,v.yvel)
	end
end

function CollisionPhysic(dt)

	if collision.timer > 0 then
		collision.timer=collision.timer-dt
	else
		collision.x=nil
		collision.y=nil
		collision.timer=0
	end

end

function checkBall()

	-- check and kill ball if reached end of life
	for i=#ball,1,-1 do
		local health=ball[i].fixture:getUserData().health
		--print("check health: "..health)
		if health<1 then 
			--print("removed")
			ball[i].body:destroy()
			table.remove(ball,i)
		end
	end
end


function checkEnemy()

	-- check and kill enemy if reached end of life
	for i=#enemy,1,-1 do
		local health=enemy[i].fixture:getUserData().health
		--print("check health: "..health)
		if health<1 then 
			--print("removed")
			enemy[i].body:destroy()
			table.remove(enemy,i)
		end
	end
end


function love.update(dt)
	checkBall()
	checkEnemy()
	NaturePhysics(dt)
	BallPhysics(dt)
	EnemyPhysics(dt)
	CollisionPhysic(dt)
	world:update(dt)
		
	if love.keyboard.isDown("right") then --press the right arrow key to push the ball to the right
	elseif love.keyboard.isDown("left") then --press the left arrow key to push the ball to the left
	end
	
	if love.keyboard.isDown("up") then --press the up arrow key to set the ball in the air
		---ball.body:setPosition(width/4,height/3)
		---ball.body:setLinearVelocity(0,0)
	end
	
	
end


function love.draw()
	
	for i,v in ipairs(ball) do
		local health=v.fixture:getUserData().health
		--print("love.draw: "..health)
		love.graphics.setColor(ball.color[health])
		love.graphics.circle('fill',v.body:getX(),v.body:getY(),v.shape:getRadius())
	end

	for i,v in ipairs(enemy) do
		local health=v.fixture:getUserData().health
		--print("love.draw: "..health)
		love.graphics.setColor(enemy.color[health])
		love.graphics.polygon("fill",v.body:getWorldPoints(v.shape:getPoints()))
	end

	love.graphics.setColor(255,0,255)
	love.graphics.polygon("fill",wall.top.body:getWorldPoints(wall.top.shape:getPoints()))
	love.graphics.polygon("fill",wall.bottom.body:getWorldPoints(wall.bottom.shape:getPoints()))
	love.graphics.polygon("fill",wall.left.body:getWorldPoints(wall.left.shape:getPoints()))
	love.graphics.polygon("fill",wall.right.body:getWorldPoints(wall.right.shape:getPoints()))
	love.graphics.setColor(255,255,255)
	love.graphics.print("Time :"..tostring(math.ceil(wind.timer)),400,100)
	if collision.x and collision.y then
		love.graphics.setColor(collision.color)
		love.graphics.circle('line',collision.x,collision.y,3)
		
	end
end

function beginContact(a, b, c)

	local au,bu=a:getUserData(),b:getUserData()
	print(au,bu)
	if au and bu then
		au.health=au.health-1
		bu.health=bu.health-1
		collision.x,collision.y=c:getPositions()
		collision.timer=1
		print(au.id,bu.id)
		print(collision.x,collision.y)
		checkBall()
		checkEnemy()
	end
end

function endContact(a, b, c)
	--cx,cy=coll:getNormal() 
end

function preSolve(a, b, c)
end

function postSolve(a, b, c, normalimpulse1, tangentimpulse1, normalimpulse2, tangentimpulse2)
end


function love.keypressed(key)
end

function love.keyreleased(key)
end
